local Queue = require("datastructures.Queue")
local Block = require("blocks.Block")
local fileutils = require("utils.fileutils")
local textureloader = require("textureloader")

local blocks = {}
blocks.all = {}

function blocks.registerClass(name, class)
  blocks.all[name] = class
end
  
function blocks.loadBlockClass(path, modName)
  local name = fileutils.extract(path)
  print("  loading block: "..name)
  
  
  local requirepath = path.."/block"
  local blockFileExists = love.filesystem.exists(requirepath..".lua")
  
  local success, blockClass
  if blockFileExists then
    success, blockClass= pcall(function() return require(requirepath) end)        
    if not success then
      print("block ["..name.."] crashed "..requirepath.." "..tostring(blockClass))
    end
    if success and not blockClass then 
      print("block ["..name.."] script hasn't returned a value!")
      success = false;
    end
  else
    success = false
  end
  
  
  if not blockClass then    
    local class = Block:new()      
    blockClass = class
  end
  
  
 
  textureloader.loadTexturesInto(path, blockClass)  
  blockClass.blockId = modName..":"..name
  if not blockClass.material or blockClass.material.name == "broken" then
    blockClass.material = blockClass.materials.texture
  end
  blocks.all[blockClass.blockId] = blockClass
end


local pool = {}
local newTime = 0
local newCount = 0
local reuseTime = 0
local reuseCount = 0
local freeTime = 0
local freeCount = 0
blocks.new = function(blockId)
  --[[
  local start = love.timer.getTime()
  local queue = pool[blockId]
  local result
  if queue and #queue > 0 then    
    result =  queue[#queue]
    queue[#queue] = nil
    reuseTime = reuseTime + love.timer.getTime()-start
    reuseCount = reuseCount + 1
    return result
  end
  
  start = love.timer.getTime()
  ]]--
  --print("creating "..blockId.." because "..(queue and ("queue length="..queue:length()) or "no queue"))
  --local blockClass = blocks.all[blockId]
  --assert(blockClass, "no registered block with blockId="..tostring(blockId))
  return blocks.all[blockId]:new()
  --assert(result, "constructor for "..tostring(blockId).." hasn't returned an object")
  --newTime = newTime + love.timer.getTime() - start
  --newCount = newCount + 1
  --return result
end

blocks.free = function(block)
  if true then return end
  local start = love.timer.getTime()
  local blockId = block.blockId
  if not pool[blockId]  then
    pool[blockId] = {}
  end
  local queue = pool[blockId]
  --if #queue > 4 * 81 * 16 * 16 then
--    print("too big "..#queue)    
  --end
  queue[#queue+1]= block
  
  freeTime = freeTime + love.timer.getTime() - start
  freeCount = freeCount + 1
 --[[ if math.random() < 0.001 then
    print("q ["..blockId.."].length = "..queue:length())
    local check = {}
    for i = queue.firstOccupied, queue.firstFree -1 do
      local item = queue[i]
      if check[item] then
        error('twice!')
      else
        check[item] = true
      end
    end
  end]]--
end

blocks.getStats = function ()
  if true then return "no stats" end
  local result = ""
  
  if newCount > 0 then
    result = result .. " newtime "..(newTime/newCount)
  end
  
  if reuseCount > 0 then
    result = result .. "\t\t reuse "..(reuseTime/ reuseCount)
  end
  
  if freeCount > 0 then
    result = result .. "\t\t free "..(freeTime / freeCount)
  end
  
  return result
end





return blocks