local Noise = require("generation.noise.Noise")
local Generator = {}
local blocks = require("blocks.blocks")
local BaseStructure = require("generation.structures.BaseStructure")
function Generator:new(seed)
  self.__index = self
  if seed then print("GENERATOR WITH SEED") end
  local instance = {
    groundBlockLayer=0,
    seed=seed
  }
  setmetatable(instance,self)
  instance:init()
  return instance
end

function Generator:init()
  
  self.scaleRatio = 1/1024
  --self.inputScale = 1/16
  --self.outputScale = 1/16
  self.noise = Noise:new(self.seed)
end

function Generator:getBlock(pt)
  return self:getBlockXY(pt.x,pt.y)
end

function Generator:getBlockXY(x,y, layerIndex)
  if not layerIndex then
    layerIndex = 0
  end
  local noise = self.noise
  
  
  
  local groundBlockLayer = self.groundBlockLayer
  local height = self.height
  local highHeight = height - groundBlockLayer
  local lowHeight = groundBlockLayer
  
  local tunnelHeight = 20
  local tunnelLowerY = groundBlockLayer - 30
  local tunnelHigherY = tunnelLowerY + tunnelHeight
  
  
  if y < tunnelLowerY then
    local structure = BaseStructure:new()
    local start = x - fmod(x, structure.width*10)
    local noiseParam = (start / (structure.width*10))*0.0381 
    local yStart = groundBlockLayer - structure.height
    --print("x "..tostring(x).." np "..tostring(noiseParam).." noise "..tostring(noise:get1d(noiseParam)))    
    if math.abs(noise:get1d(noiseParam))<0.001 and (x - start) < structure.width and (y- yStart)>=0 and (y - yStart ) < structure.height then
      local result =  structure:getBlock(x-start+1, y - yStart+1, layerIndex)
      if result then
        return result
      end
    end 
  end
    
  local noiseValue=0
  local noise2dValue = 0
  for power = 1,8 do
    local inputScale = 1/ 2 ^ power
    local outputScale  = self.scaleRatio / inputScale
    noiseValue = noiseValue + noise:get1d(x*inputScale)*outputScale    
    noise2dValue = noise2dValue + noise:get2d(x*inputScale, y*inputScale)/8
  end
  --local inputScale = 1/ 2^4
  
  
  
  
        
  local yMultiplier = noiseValue < 0 and lowHeight or highHeight
  local yLevel = groundBlockLayer +  math.floor(noiseValue * yMultiplier)
  
  local isWater = yLevel > groundBlockLayer
  
  local block = nil      
  local id = nil
  
  if y == tunnelLowerY or y == tunnelHigherY then
    id = "concrete"
  elseif layerIndex < 2 and y > tunnelLowerY and y < tunnelHigherY then
    if layerIndex == 0 then
      id = "sky"
    else
      id = "glass"
    end
  elseif y == yLevel-1 and yLevel == groundBlockLayer+1 then
    id = "bamboo"
  elseif  y == yLevel then
    id= isWater and "gravel" or "grass"    
  elseif y < yLevel then
    if isWater and y > groundBlockLayer then
      id="water"
    else
      id="sky"
    end        
  else 
    if noise2dValue < 0.3 then
      id="earth"
    elseif noise2dValue < 0.4 then
      id="ore_iron"
    elseif noise2dValue < 0.5  then
      id="ore_gold"
    else
      id="ore_diamond"
    end
  end
  
  --[[if id=="sky" then
    if self:getBlockXY(x, y+1) then
      if self:getBlockXY(x+1,y+1).blockId == "water" or self:getBlockXY(x-1, y+1).blockId == "water" then
        id="bamboo"
      end
    end
  end]]--
  
  block = blocks.new("core:"..id)
  return block
end

return Generator