local M = {}

local Point = {}
Point.x = 0
Point.y = 0
local debugPoints = false
if debugPoints then
  Point.__index = function(t,k)
    if k =="x" or k=="y" then
      return t.storage[k]
    else
      return Point[k]
    end
  end
  
  Point.__newindex = function(t,k,v)
    if k == "x" or k =="y" then
      if v ~= v or v == 0/0 then
        
        mobdebug.on()
        local debug = true
        error("point shouldn't contain nans")
      end
      t.storage[k] = v
    else
      rawset(t,k,v)
    end
  end
  
  function Point:new(x, y)  
    local instance = {x=x or 0, y=y or 0}  
    setmetatable(instance,self)
    instance.storage = {x=x, y=y}
    return instance
  end
  
else
  Point.__index = function(t,k)
    if k=="x" then 
      return rawget(t,1) or 0
    elseif k=="y"  then
      return rawget(t,2) or 0
    else 
      return Point[k]
    end
  end
  
  Point.__newindex = function(t,k,v)
    if k =="x" then 
      rawset(t,1,v)
    elseif k=="y" then
      rawset(t,2, v)
    else
      rawset(t,k,v)
    end
  end
  --Point.__index = Point
  function Point:new(x, y)    
   
    return setmetatable( {x or 0,y or 0}, self)  
  end
end


function Point:__mul(number)
  return setmetatable({self[1]*number,self[2]*number},Point)
end

function Point:__add(other)
  return setmetatable({self[1]+other[1],self[2]+other[2]},Point)
end

function Point:__sub(other)
  return setmetatable({self[1]-other[1],self[2]-other[2]},Point)
end

function Point:__tostring()
  return ""..self.x..","..self.y
end

function Point:set(other)
  self[1]=other[1]
  self[2]=other[2]
  return self
end

function Point:set2(x,y)
  self[1] = x
  self[2] = y
  return self
end

function Point:length()
  return math.sqrt(self.x*self.x+self.y*self.y)
end

function Point:lengthSquared()
  return self.x*self.x + self.y*self.y
end

function Point:scaledTo(newLength)
  local originalLength = self:length()
  if originalLength  == 0 then
    return Point:new(newLength,0)
  else
    local ratio = newLength/originalLength
    return self * ratio
  end
end

function Point:clone()
  return setmetatable({self[1],self[2]},Point)
end

function Point:cappedLength(maxLength)
  if maxLength  < self:length() then
    return self:scaledTo(maxLength)
  else
    return self:clone()
  end
end


function Point:modInsideRect(topLeft, bottomRight)
  local newX = self.x
  local newY = self.y
  local w = bottomRight.x - topLeft.x
  local h = bottomRight.y - topLeft.y
  
  while newX < topLeft.x do 
    newX = newX + w
  end
  while newY < topLeft.y do
    newY = newY + h
  end
  while newX > bottomRight.x do
    newX = newX - w
  end
  while newY > bottomRight.y do
    newY = newY - h
  end
  return Point:new(newX,newY)  
  
end
  
function Point:round()
  local pos = Point:new(math.floor(self.x+0.5),math.floor(self.y+0.5))
   assert(pos.x==math.floor(pos.x))
  assert(pos.y==math.floor(pos.y))
  return pos
end

function Point:assertInt()
  assert(self[1]==math.floor(self[1]))
  assert(self[2]==math.floor(self[2]))
  return self
    
end
  

function Point:swapped()
  return Point:new(self.y, self.x)
end

function Point:equals(other)
  return self[1]==other[1] and self[2]==other[2]
end


function M.createPointCollection(coordsList)
  local result = {}
  for i = 1,#coordsList,2 do
    local x = coordsList[i]
    local y = coordsList[i+1]
    table.insert(result,Point:new(x,y))
  end
  return result
end


M.Point = Point
M.deltas = {}
M.deltas.up = Point:new(0,-1)
M.deltas.down = Point:new(0,1)
M.deltas.left = Point:new(-1,0)
M.deltas.right = Point:new(1,0)



return M



