local ffi = require("ffi")

ffi.cdef[[
  typedef struct {
    char *fpos;
    void *base;
    unsigned short handle;
    short flags;
    short unget;
    unsigned long alloc;
    unsigned short buffincrement;
  } FILE;

  FILE *fopen(const char *filename, const char *mode);
  int fprintf(FILE *stream, const char *format, ...);
  int fclose(FILE *stream);
  int fseek(FILE *stream, long int offset, int whence);
  long int ftell(FILE *stream);
  void *malloc(size_t size);
  void free (void* ptr);
  size_t fread(void *ptr, size_t size, size_t nmemb, FILE *stream);
  
  typedef char* pchar;
]]

local C = ffi.C
local SEEK_END = 2
local SEEK_CUR = 1
local SEEK_SET = 0



local FILE_NOT_EXISTS = 2
local p = function(message)
  if true then return end
  print(message)
  io.stdout:flush()
end
local inputChannel = love.thread.getChannel("loadFileChannelParams")    
local outputChannel = love.thread.getChannel("fileChannelResult")
p("load thread ready!")

repeat
  local fileName, callBackIndex = unpack(inputChannel:demand())    
  
  local data
  
  local f = C.fopen(fileName, "rb")
  if f ~= nil then
    --io.stderr:write(tostring(f).."\n")
    --io.stderr:flush()
    C.fseek(f,0, SEEK_END)
    local fsize = C.ftell(f)
    C.fseek(f,0, SEEK_SET)
    
    local charptr = ffi.cast("pchar",C.malloc(fsize+1))
    C.fread(charptr, fsize, 1, f)
    C.fclose(f)    
    charptr[fsize] = 0
    
    data = ffi.string(charptr)
    C.free(charptr)
  end
  outputChannel:push({callBackIndex, data})
until false