local M = {}
function M.extractExtension(path)
  local nameWithExtension = M.extract(path):reverse()
  local extensionEnd = nameWithExtension:find(".",nil, true)-1   
  local extensionInverted = nameWithExtension:sub(0,extensionEnd)
  local extension = extensionInverted:reverse()
  -- print("path "..path.."=>"..nameWithExtension.."=>("..fileNameStart..")"..name)
  return extension
end
function M.extractName(path)
    local nameWithExtension = M.extract(path):reverse()
    local fileNameStart = nameWithExtension:find(".",nil, true)+1   
    local nameInverted = nameWithExtension:sub(fileNameStart)
    local name = nameInverted:reverse()
    -- print("path "..path.."=>"..nameWithExtension.."=>("..fileNameStart..")"..name)
    return name
end

function M.extract(path)
  local fileNameBack  = path:reverse()    
  local fileNameEnd = fileNameBack:find("/")
  if not fileNameEnd then
    return path
  else
    local nameInverted = fileNameBack:sub(1, fileNameEnd-1)
    local name = nameInverted:reverse()
    return name
  end
end

function M.forChildPaths(parentPath, handler, filter)
  if type(filter)=="string" then
    local originalFilter = filter
    filter = function(path) 
      return path:match(originalFilter)~=nil; 
    end
  elseif type(filter)~="function" then  
    filter = function() return true; end  
  end
  for i, childName in ipairs(love.filesystem.getDirectoryItems(parentPath)) do
    
    local childPath = parentPath.."/"..childName
    if filter(childName) then
      handler(childPath)
    end
  end
end


function M.getModsRoot()
  return "assets/mods";
end

function M.getSavesRoot()    
  return "assets/saves";
end

function M.getChunksRoot()
  local result =  M.getSavesRoot().."/chunks"
  love.filesystem.createDirectory(result)
  return result
end

function M.pathToId(path)
  local prefix = M.getModsRoot()
  if path:find(prefix) then
    path = path:sub(prefix:len()+1)
  end
  local suffix = M.extractExtension(path)
  if suffix then
    path = path:sub(1, path:len()-suffix:len())
  end
  path = path:gsub("/",":")
  return path
end

function M.getChunkFileNameById(chunkId)
  return M.getChunksRoot().."/"..chunkId..".txt"
end

local asyncFileCounter = 1
local asyncFileCallBacks = {}
local _loadThread
local function getLoadThead()
  if not _loadThread then
    _loadThread = love.thread.newThread("threads/loadThread.lua")
    _loadThread:start()
  end
  if _loadThread:getError() then
     io.stderr:write("Load thread error: "..tostring(_loadThread:getError()).."\n")
  end
  if not _loadThread:isRunning() then
     io.stderr:write("Load thread not running!\n")
  end
  return _loadThread
end

local _saveThread
local function getSaveThread()
  if not _saveThread then
    _saveThread = love.thread.newThread("threads/saveThread.lua")
    _saveThread:start()
  end
  if _saveThread:getError() then
    io.stderr:write("save thread error: "..tostring(_saveThread:getError()).."\n")
  end
  if not _saveThread:isRunning() then
    io.stderr:write("save thread not running!\n")
  end
  return _saveThread
end



function M.loadFileAsync(path, callBack)
  local loadThread = getLoadThead()
  local paramsChannel = love.thread.getChannel("loadFileChannelParams")  
  asyncFileCallBacks[asyncFileCounter] = callBack
  paramsChannel:push({path, asyncFileCounter})  
  asyncFileCounter = asyncFileCounter + 1
end

function M.handleAsyncResults()
  local resultChannel = love.thread.getChannel("fileChannelResult")
  repeat
    local message = resultChannel:pop()
    if message then      
      local callBackIndex,data = unpack(message)      
      local callBack = asyncFileCallBacks[callBackIndex]
      if callBack  then
        callBack(data)
        asyncFileCallBacks[callBackIndex] = nil
      end
    end
  until not message

end


function M.saveFileAsync(path, data, callBack)
  local saveThread = getSaveThread()
  local paramsChannel = love.thread.getChannel("saveFileChannelParams")  
  if paramsChannel:getCount() > 1000 then
    print("BAD THING channel queue length = "..paramsChannel:getCount())
  end
  asyncFileCallBacks[asyncFileCounter] = callBack
  paramsChannel:push({path, data,asyncFileCounter})  
  asyncFileCounter = asyncFileCounter + 1
end

return M